#!/bin/bash

# stockfish command line (optional xboard) simple shell script

xboard_exists () {
  type xboard >/dev/null 2>&1
  return $?
}

if [[ -z $1 ]]; then
  NAME=$(basename $0 .sh)
  UCI=~/.$NAME.uci
  POSITION_PREFIX="position startpos moves"
  if [[ ! -f $UCI ]]; then
    echo "uci" > $UCI
    echo "setoption name Skill Level value 0" >> $UCI
    echo "isready" >> $UCI
    echo "ucinewgame" >> $UCI
    echo $POSITION_PREFIX >> $UCI
    echo "go movetime 1000" >> $UCI
    echo "go" >> $UCI
  fi
  EXPORT_PGN=~/Downloads/$NAME.pgn
  GREP="grep --extended-regexp --only-matching"
  SED="sed --regexp-extended --in-place"
  MOVE_EXP="([a-h][1-8]){2}[qrbn]?"
  clear
  while [[ -f $UCI ]]; do
    MOVES=$($GREP "$POSITION_PREFIX.+" $UCI | $GREP "($MOVE_EXP[ ]?)+")
    if [[ -n $MOVES ]]; then
      echo ": $MOVES"
    fi
    read -p "> " USER_MOVE
    if [[ $USER_MOVE =~ ^$MOVE_EXP$ ]] || [[ -z $USER_MOVE ]]; then
      if [[ -n $USER_MOVE ]]; then
        $SED --expression="s/$POSITION_PREFIX.*/\0 $USER_MOVE/" $UCI
      fi
      BEST_MOVE=$(stockfish < $UCI | $GREP --max-count 1 "bestmove $MOVE_EXP" | $GREP $MOVE_EXP)
      if [[ -n $BEST_MOVE ]]; then
        $SED --expression="s/$POSITION_PREFIX.*/\0 $BEST_MOVE/" $UCI
        echo -e "  \e[1;32m$BEST_MOVE\e[0m"
      else
        break
      fi
    else
      case $USER_MOVE in
        "new")
          $SED --expression="s/($POSITION_PREFIX).+/\1/" $UCI
          clear
        ;;
        "undo")
          $SED --expression="s/($POSITION_PREFIX.+)( $MOVE_EXP){2}$/\1/" $UCI
        ;;
        "list")
          if [[ -n $MOVES ]]; then
            clear
            for LIST_MOVE in $MOVES; do
              echo $LIST_MOVE
              read -n 1 -s LIST_KEY
              case $LIST_KEY in
                ")" | "]" | "q")
                  if [[ ! $LIST_KEY == "q" ]]; then
                    read -n 1 -s -p "$LIST_KEY? " CUT_KEY
                    if [[ $CUT_KEY == "y" ]]; then
                      if [[ $LIST_KEY == ")" ]]; then
                        $SED --expression="s/($POSITION_PREFIX.*) $LIST_MOVE.*/\1/" $UCI
                      else
                        $SED --expression="s/($POSITION_PREFIX.* $LIST_MOVE).*/\1/" $UCI
                      fi
                    fi
                  fi
                  break
                ;;
              esac
            done
          fi
          clear
        ;;
        "save" | "export" | "ex")
          if [[ -n $MOVES ]]; then
            if [[ $USER_MOVE == "save" ]]; then
              PGN=~/Documents/$USER-$NAME.pgn
              if [[ -f $PGN ]]; then
                echo >> $PGN
              fi
              echo "[Event \"Stockfish Chess Game\"]" >> $PGN
              echo "[Site \"$HOSTNAME\"]" >> $PGN
              echo "[Date \"$(date +"%Y.%m.%d")\"]" >> $PGN
              echo >> $PGN
              echo $MOVES >> $PGN
            else
              echo $MOVES > $EXPORT_PGN
              if xboard_exists; then
                pushd $(dirname $EXPORT_PGN) > /dev/null
                xboard $(basename $EXPORT_PGN) &
                popd > /dev/null
              fi
            fi
          fi
        ;;
        "reset")
          rm $UCI
        ;;
        "q" | "quit")
          break
        ;;
        *)
          vi $UCI
        ;;
      esac
    fi
  done
  clear
  rm -f $EXPORT_PGN
else
  if xboard_exists; then
    if [[ -f ~/volume.sh ]]; then
      ~/volume.sh low
    fi
    XBOARD="xboard -fcp"
    case $1 in
      "fairymax" | "max" | "fairy")
        $XBOARD fairymax
      ;;
      *)
        $XBOARD stockfish -fUCI -adapterCommand "polyglot -noini -ec \"%fcp\" -ed \"%fd\" -uci \"Skill Level=0\""
      ;;
    esac
  fi
fi
